require 'date'
class GraphController < ApplicationController
  # produce a graph of votes per day during an election
  def votes_per_day
    @election = Election.find(params[:id])
    data, labels = get_votes_per_day_data(@election)
    
    line = Gruff::Line.new("700x400")
    line.theme = { :background_colors => ['#73BF26', '#ffffff'] }
    line.title = "Voters Per Day"
    line.font = File.expand_path('/usr/X11R6/lib/X11/fonts/TTF/Vera.ttf',
                                 RAILS_ROOT)

    line.data("#{@election.name}", data )
    line.labels = labels

    line.x_axis_label = "Date"
    line.y_axis_label = "Number of Votes"
    line.minimum_value = 0.0

    line.draw
    send_data(line.to_blob, :disposition => 'inline', :type => 'image/png')
  end
 
  def votes_per_interval
    @election = Election.find(params[:id])
    data, labels = get_votes_per_interval_data(@election)
    
    line = Gruff::Line.new("700x400")
    line.theme = { :background_colors => ['#73BF26', '#ffffff'] }
    line.title = "Voters Over Time"
    line.font = File.expand_path('/usr/X11R6/lib/X11/fonts/TTF/Vera.ttf',
                               RAILS_ROOT)
    
    line.data("#{@election.name}", data )
    line.labels = labels
    
    line.x_axis_label = "Intervals"
    line.y_axis_label = "Number of Votes"
    line.minimum_value = 0.0
                               
    line.draw
    send_data(line.to_blob, :disposition => 'inline', :type => 'image/png')  
  end
 
 private 
 
  # generate the data and labels for each graph
  def get_votes_per_day_data(election)
    voter_days = Array.new
    unique_days = Array.new
    total_per_day = Array.new
    election_days = Hash.new
    
    #turn election startdate into date object, and create the range of election
    startdate = Date.parse(election.startdate.to_s)
    election_range = startdate..Date.today
    
    # create a hash with all the dates of the election in String format
    # referenced by their order in the election
    election_range.each_with_index do |day, index|
      election_days[index] = day.to_s
    end
    
    # Now I need to create an array with all the times votes were made
    election.votes.each do |vote|
        voter_days << Date.parse(vote.time.to_s)
    end
    voter_days.sort!
    
    # Now I need to count how many times each each date appears in voter_days,
    # and put that number into a votes_per_day array, the 'data' for the graph    
    #Create an array of unique days from voter_days
    voter_days.each do |day|
      unless unique_days.any? {|date| date.eql?(day)}
        unique_days << day
      end
    end
    unique_days.sort!
    
    #find all dates where those days = date at current index, put size of returned
    #array into total_per_day
    unique_days.each_with_index do |date, index|
      total_per_day << (voter_days.select {|day| day.eql?(date)}).size
    end    

    # return the data and the labels
    return total_per_day, election_days
   
  end
  
  def get_votes_per_interval_data(election)
    labels_hash = Hash.new
    buckets = Hash.new
    total_per_interval = Array.new
    
    starttime = election.startdate
    timedelta = Time.now - starttime
    numcols = 10
    interval_length = timedelta/numcols
    
    # Make a hash, buckets, indexed by time intervals and containing empty arrays
    # The time object must come first in addition! 
    # i would start at 0, i+1 goes from 0 up till numcols
    numcols.times {|i| buckets[starttime + ((i+1)*interval_length)] = []}
     
    # Put votes into bucket according to the time interval to which they belong,
    # referenced by their key
    # Will build a graph over time, as each successive interval wil lhave more
    # vote objects  
    election.votes.each do |vote|
      buckets.keys.sort.each do |inter|
        if vote.time < inter
          buckets[inter] << vote
        end
      end
    end
  
    total_per_interval = buckets.keys.sort.collect {|key| buckets[key].size}
    
    # Create the hash for the labels. Each graph has ten columns, and three
    # will be labeled
    if timedelta < 2.hours #under two hours use minutes for labels
      labels_hash[0] = starttime.min.to_s
      labels_hash[(numcols/2)-1] = (starttime + (timedelta/2)).min.to_s
      labels_hash[numcols-1] = Time.now.min.to_s
    elsif timedelta < 2.days #more than 2 hours means use hours for labels
      labels_hash[0] = starttime.hour.to_s
      labels_hash[(numcols/2)-1] = (starttime + (timedelta/2)).hour.to_s
      labels_hash[numcols-1] = Time.now.hour.to_s
    else #more than 2 days means use dates for labels
      labels_hash[0] = (Date.parse(starttime.to_s)).to_s
      labels_hash[(numcols/2)-1] = (Date.parse(starttime + (timedelta/2))).to_s
      labels_hash[numcols-1] = (Date.today).to_s
    end
    
    # Make sure to return an array for data and hash for labels
    return total_per_interval, labels_hash   
  end

end
